""" Addon Information """
# ------------
# EventScripts Script-Addon:    BackRaw's Deathmatch (BDM, bdm.py, 17.3kb)
# Author:                       BackRaw - http://forums.eventscripts.com/profile.php?mode=viewprofile&u=13683
#                                         http://addons.eventscripts.com/addons/user/13683
# Version:                      3.0
# ------------

""" Imports """
# ------------
# EventScripts Core-Module                - ES-functionality
import es
# Gamethread                              - Delayed functions
import gamethread
# Popuplib                                - Create popups
import popuplib
# Source Python Extensions                - Spawning players, safely removing weapons and entities
from spe.games.cstrike import respawn
from spe.games.shared import removeEntityByIndex
# Random                                  - Randomized choices
from random import choice as rChoice
# Config Object                           - Configuration-File: ./addons/eventscripts/bdm/bdm_config.ini
from configobj import ConfigObj
# OS                                      - Path seperator
from os import sep, remove as removeFile
# os.path                                 - Check file existens
from os.path import isfile
# Psyco                                   - Optimize this script
import psyco
psyco.full()
# ------------

""" Global Variables """
# ------------
# This player-dictionary will hold every player in the class "DeathMatch".
# It will return the player's chosen secondary and primary weapons and the chosen menu.
# ------------
player = {}
# We need es.getAddonPath() more often.
path = es.getAddonPath('bdm')

""" Configuration """
# ------------
# This will get the config values of types str, float, and int for easier handling.
# ------------
class config(object):
    def __init__(self, name=False, section=False):
        self.file = ConfigObj(path + '/bdm_config.ini')
        self.name = name
        self.section = section
    def getString(self):
        if self.section:
            return self.file[self.section][self.name]
        return self.file[self.name]
    def getFloat(self):
        return float(self.getString())
    def getInt(self):
        return int(self.getString())

""" DeathMatch """
# ------------
# Deathmatch functionality.
# Will strip the players off their weapons, give the players their chosen weapons and so on... <- "Death Match" ;)
# ------------
class DeathMatch(object):
    """ When initialized, tell the Class to use the given UserID the whole time until it doesn't exist any more. Set secondary and primary weapons and the chosen menu to zero. """
    def __init__(self, userid):
        self.userid = userid
        self.sec = self.prim = self.chosen = 0
    # es_fire wrapper for easier using
    def fire(self, cmd):
        es.server.queuecmd('es_xfire %s %s' % (self.userid, cmd))
    # es_give wrapper for easier using
    def give(self, cmd):
        es.server.queuecmd('es_xgive %s %s' % (self.userid, cmd))
    # function wrapper, de-/activates buyzone and such functions
    def funcs(self, arg):
        arg = arg.capitalize()
        for x in ('buyzone', 'hostage_rescue', 'bomb_target'):
            self.fire('func_%s %s' % (x, arg))
        if arg == 'Disable':
            self.fire('hostage_entity Kill')
    # strip-players function
    def strip(self):
        self.fire('player_weaponstrip kill')
        self.give('player_weaponstrip')
        self.fire('player_weaponstrip strip')
        gamethread.delayed(0, self.giveWeapons)
    # actually give the weapons
    def giveWeapons(self):
        for x in ('item_assaultsuit', 'weapon_knife'):
            self.give(x)
        if self.chosen != 'random' or es.getplayersteamid(self.userid) != 'BOT':
            if self.sec:
                gamethread.delayed(0, self.give, 'weapon_%s' % self.sec)
            if self.prim:
                gamethread.delayed(0, self.give, 'weapon_%s' % self.prim)
        else:
            self.randomWeapons()
        gamethread.delayed(0, self.give, 'weapon_hegrenade')
        if config('flashbang', 'grenades').getInt():
            gamethread.delayed(0, self.give, 'weapon_flashbang')
        if config('smokegrenade', 'grenades').getInt():
            gamethread.delayed(0, self.give, 'weapon_smokegrenade')
    # randomized weapons
    def randomWeapons(self):
        self.strip()
        gamethread.delayed(0, self.giveRandom)
    def giveRandom(self):
        for x in ('weapon_%s' % rChoice(config('secondary', 'random').getString()), 'weapon_%s' % rChoice(config('primary', 'random').getString())):
            self.give(x)
    # check if player is in stuck with another player
    def checkStuck(self):
        loc = es.getplayerlocation(self.userid)
        for x in filter(lambda x: str(x) != self.userid and es.getplayerlocation(x) == loc, es.getUseridList()):
            self.setSpawnLocation()
    # set the spawn location by choosing a spawn-point
    def setSpawnLocation(self):
        curmap = str(es.ServerVar('eventscripts_currentmap'))
        f = path + '%sspawnpoints%s%s.txt' % (sep, sep, curmap)
        try:
            a = open(f)
            b = rChoice(a.read().split('\n')).replace(' ', '').split(',')
            a.close()
            es.server.queuecmd('es_xsetpos %s %s %s %s' % (self.userid, b[0], b[1], b[2]))
            gamethread.delayed(.5, self.checkStuck)
        except:
            if isfile(f):
                self.setSpawnLocation()
            else:
                raise IOError, 'File for map "%s" does not exist!' % curmap
    # player_spawn
    def spawn(self):
        self.funcs('disable')
        if self.chosen == 'random' or es.getplayersteamid(self.userid) == 'BOT':
            self.randomWeapons()
        elif self.chosen != 'random':
            if self.chosen:
                if self.chosen == 'display':
                    gamethread.delayed(.2, menu.send, self.userid)
                self.strip()
            else:
                gamethread.delayed(.2, menu.send, self.userid)
        else:
            gamethread.delayed(.2, menu.send, self.userid)
        self.setSpawnLocation()
    # main-menu
    def _menu(self, choice):
        self.chosen = choice
        if not self.chosen:
            sec.send(self.userid)
        else:
            if self.chosen != 'random':
                self.strip()
            else:
                self.randomWeapons()
    # secondary-menu
    def _sec(self, choice):
        self.sec = choice
        prim.send(self.userid)
    # primary-menu
    def _prim(self, choice):
        self.prim = choice
        self.strip()
        #menu.send(self.userid)

""" Instances - bdmPlayer (DeathMatch) """
# ------------
# We want to call this instead of calling the DeathMatch class itself.
# ------------
def bdmPlayer(userid):
    if es.exists('userid', userid):
        return DeathMatch(userid)
    raise TypeError, 'UserID %s does not exist!' % userid

""" End of Class Management - Start wrappers """
# ------------
# Wrapper for removing weapons and ragdolls es-only.
# ------------
def remove(userid, weapon=0):
    pl = player[str(userid)]
    pl.fire('cs_ragdoll Kill')
    if weapon:
        pl.fire('%s Kill' % weapon)
    else:
        entlist = es.createentitylist()
        for x in filter(lambda x: entlist[x]['classname'].startswith('weapon_') and es.getindexprop(x, 'CBaseEntity.m_hOwnerEntity') == -1, entlist):
            removeEntityByIndex(x)

""" End wrappers - Start Event-Functions """
# ------------
# Add the connected player to the player-dictionary.
# ------------
def player_activate(ev):
    userid = ev['userid']
    if not userid in player:
        player[userid] = bdmPlayer(userid)

# ------------
# If the player is on a team, send the menu or strip the player.
# ------------
def player_spawn(ev):
    userid = ev['userid']
    if int(ev['es_userteam']) > 1 and userid in player:
        player[userid].spawn()

# ------------
# Give the player a HE-Grenade that threw one.
# ------------
def hegrenade_detonate(ev):
    if config('giveback', 'grenades').getInt():
        userid = ev['userid']
        if not es.getplayerprop(userid, 'CBasePlayer.pl.deadflag'):
            bdmPlayer(userid).give('weapon_hegrenade')

# ------------
# Give Flashbang back if set to 1 in the config file
# ------------
def flashbang_detonate(ev):
    userid = ev['userid']
    if not es.getplayerprop(userid, 'CBasePlayer.pl.deadflag') and config('flashbang', 'grenades').getInt() and config('giveback', 'grenades').getInt():
        bdmPlayer(userid).give('weapon_flashbang')

# ------------
# Give Smokegrenade back if set to 1 in the config file
# ------------
def smokegrenade_detonate(ev):
    userid = ev['userid']
    if not es.getplayerprop(userid, 'CBasePlayer.pl.deadflag') and config('smokegrenade', 'grenades').getInt() and config('giveback', 'grenades').getInt():
        bdmPlayer(userid).give('weapon_smokegrenade')

# ------------
# Spawn the player if he/she's dead on team choose
# ------------
def player_team(ev):
    def checkDead(userid):
        if es.getplayerprop(userid, 'CBasePlayer.pl.deadflag'):
            nt.spawnPlayer(userid, True)
    gamethread.delayed(3, checkDead, ev['userid'])

# ------------
# If FF is on, give back health to attacked team-mates.
# ------------
def player_hurt(ev):
    userid = ev['userid']
    if int(es.ServerVar('mp_friendlyfire')) and ev['es_userteam'] == ev['es_attackerteam']:
        es.setplayerprop(userid, 'CBasePlayer.m_iHealth', es.getplayerprop(userid, 'CBasePlayer.m_iHealth') + int(ev['dmg_health']))

# ------------
# Remove weapons and ragdolls and spawn the dead player.
# ------------
def player_death(ev):
    userid = int(ev['userid'])
    attacker = ev['attacker']
    respawnTime = config('respawn', section=False).getFloat()
    if not es.getplayerprop(attacker, 'CBasePlayer.pl.deadflag'):
        bdmPlayer(attacker).give('info_null')
        if int(ev['headshot']):
            es.setplayerprop(attacker, 'CBasePlayer.m_iHealth', 100)
    gamethread.delayed(respawnTime, remove, userid)
    gamethread.delayed(respawnTime, respawn, userid)

# ------------
# Remove the player from the player-dictionary.
# ------------
def player_disconnect(ev):
    userid = ev['userid']
    if userid in player:
        del player[userid]

# ------------
# Activate "round never end"...
# ------------
def round_start(ev):
    gamethread.delayedname(float(ev['timelimit']) - 5, 'endRound', es.server.queuecmd, ('es_xflags remove cheat endround;endround;es_xflags add cheat endround'))

# ------------
# Cancel "round never end"...
# ------------
def round_end(ev):
    gamethread.cancelDelayed('endRound')

""" End Event-Functions - Filters """

""" Client Command Filter """
# ------------
# Remove weapons and ragdolls as soon as somebody drops a weapon intentionally.
# ------------
def commandFilter(userid, args):
    if args[0].lower() == 'drop':
        weapon = es.createplayerlist(userid)[int(userid)]['weapon']
        gamethread.delayed(1, remove, (userid, weapon))
    return True

""" Say Command Filter """
# ------------
# Open up the spawn-menu and don't show the message typed (filter for on-the-fly configuration)
# ------------
def sayFilter(userid, text, teamonly):
    global loc, sm, curmap
    if config('spawn-point saycommand').getString() in text.strip('"'):
        if es.getplayersteamid(userid) in config().file['admins']:
            curmap = es.ServerVar('eventscripts_currentmap')
            loc = str(es.getplayerlocation(userid)).replace('(', '').replace(')', '')
            name = 'bdm_spawnmenu'
            if popuplib.exists(name):
                popuplib.delete(name)
            sm = popuplib.create(name)
            sm.addline('BDM - SpawnMenu')
            sm.addline('------------------')
            sm.addline(' ')
            sm.addline('Current Map: %s' % curmap)
            sm.addline('File for the map %s.' % str(isfile(path + '%sspawnpoints%s%s.txt' % (sep, sep, curmap))).replace('True', 'exists').replace('False', "doesn't exist"))
            sm.addline('------------------')
            sm.addline(' ')
            sm.addline('Current Location:')
            sm.addline(loc)
            sm.addline(' ')
            sm.addline('->1. Create Spawn-Point')
            sm.addline('->2. Refresh Location')
            sm.addline('->3. Remove Spawn-Point File')
            sm.addline(' ')
            sm.addline('------------------')
            sm.addline('0. Close')
            sm.menuselect = Spawn
            sm.send(userid)
        else:
            es.tell(userid, '#multi', '#green[#lightgreenBDM#green] #defaultYou are not authorized to run the command #greenspawn#default!')
        return (None, None, None)
    return (userid, text, teamonly)

""" Filters - Start menu-functions """
# ------------
# Menu-handler for spawn-point management.
# ------------
def Spawn(userid, choice, popupid):
    def spawn_edit(loc, open_method):
        a = open(path + '%sspawnpoints%s%s.txt' % (sep, sep, curmap), open_method + 'b')
        a.write('%s\n' % loc)
        a.close()
    global loc
    if choice != 10:
        _file = path + '%sspawnpoints%s%s.txt' % (sep, sep, curmap)
        if choice == 1:
            try:
                spawn_edit(loc, 'a')
            except:
                spawn_edit(loc, 'w')
            pos = loc.split(',')
            es.tell(userid, '#multi', '#green[#lightgreenBDM#green] #lightgreen%s#default: Created Spawn-Point at:' % curmap)
            es.tell(userid, '#multi', '#green%s#default, #green%s#default, #green%s' % (pos[0], pos[1], pos[2]))
        elif choice == 2:
            loc = str(es.getplayerlocation(userid)).replace('(', '').replace(')', '')
            sm.modline(9, loc)
        elif choice == 3:
            if isfile(_file):
                removeFile(_file)
                es.tell(userid, '#multi', '#green[#lightgreenBDM#green] #defaultRemoved file #lightgreen%s.txt' % curmap)
            else:
                es.tell(userid, '#multi', '#green[#lightgreenBDM#green] #defaultFile #lightgreen%s.txt #defaultdoes not exist!' % curmap)
        sm.modline(5, 'File for the map %s.' % str(isfile(_file)).replace('True', 'exists').replace('False', "doesn't exist"))
        sm.send(userid)

# ------------
# Call the class-function _menu...
# ------------
def Menu(userid, choice, popupid):
    player[str(userid)]._menu(choice)

# ------------
# Call the class-function _sec...
# ------------
def Sec(userid, choice, popupid):
    player[str(userid)]._sec(choice)

# ------------
# Call the class-function _prim...
# ------------
def Prim(userid, choice, popupid):
    player[str(userid)]._prim(choice)

""" End menu-functions - Start say-command "guns" """
# ------------
# Guns menu call.
# ------------
def guns():
    menu.send(es.getcmduserid())

""" End say-command "guns" - Start main-control functions """
# ------------
# Create menus, add players to the player-dictionary, set public variable
# ------------ 
def load():
    global menu, sec, prim
    es.ServerVar('bdm_ver', '3.0').makepublic()
    menu = popuplib.easymenu('bdm_mainmenu', '_popup_choice', Menu)
    menu.settitle('Choose an Option:')
    menu.addoption(0, 'New Equipment')
    menu.addoption('display', 'Display this menu again.')
    menu.addoption('never', 'Never display this menu again.')
    menu.addoption('random', 'Random Weapons')
    sec = popuplib.easymenu('sec', '_popup_choice', Sec)
    sec.settitle('Choose a Secondary Weapon')
    sec.addoption('glock', 'Glock 18C')
    sec.addoption('usp', 'USP')
    sec.addoption('p228', 'P228 Compact')
    sec.addoption('deagle', 'Deagle')
    sec.addoption('fiveseven', 'ES FiveSeven')
    sec.addoption('elite', 'Elite')
    sec.addoption(0, 'None')
    prim = popuplib.easymenu('prim', '_popup_choice', Prim)
    prim.settitle('Choose a Primary Weapon')
    prim.addoption('m4a1', 'Maverick M4A1')
    prim.addoption('ak47', 'AK-47')
    prim.addoption('awp', 'Artic Warfare Police (AWP)')
    prim.addoption('scout', 'Steyr Scout')
    prim.addoption('sg552', 'SG-552 Commando')
    prim.addoption('galil', 'IMI Galil')
    prim.addoption('famas', 'Famas')
    prim.addoption('aug', 'Steyr AUG')
    prim.addoption('m249', 'M249 Para')
    prim.addoption('ump45', 'Universal Mashine Pistol (UMP 45)')
    prim.addoption('mp5navy', 'MP5-Navy')
    prim.addoption('m3', 'M3 Super 90 Entry')
    prim.addoption('xm1014', 'XM1014 (M4 Super 90)')
    prim.addoption('tmp', 'Tactical Mashine Pistol (TMP)')
    prim.addoption('mac10', 'Ingram MAC-10')
    prim.addoption('p90', 'P90')
    prim.addoption(0, 'None')
    es.addons.registerClientCommandFilter(commandFilter)
    es.addons.registerSayFilter(sayFilter)
    es.regsaycmd('guns', 'bdm/guns')
    for x in filter(lambda x: str(x) not in player, es.getUseridList()):
        x = str(x)
        player[x] = bdmPlayer(x)

# ------------
# Delete menus, remove players from the player-dictionary
# ------------
def unload():
    menu.delete()
    sec.delete()
    prim.delete()
    player.clear()
    gamethread.cancelDelayed('endRound')
    es.addons.unregisterClientCommandFilter(commandFilter)
    es.addons.unregisterSayFilter(sayFilter)
    es.unregsaycmd('guns')
    for userid in es.getUseridList():
        bdmPlayer(userid).funcs('enable')

""" End Code """
